/*
 * This file belongs to the XQuark distribution. Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307. You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.servlet.bridge;

import java.io.*;
import java.net.MalformedURLException;
import java.util.HashMap;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xquark.bridge.Mapper;
import org.xquark.bridge.Mapping;
import org.xquark.bridge.XQBridge;
import org.xquark.serialize.XMLSerializer;
import org.xquark.servlet.util.FileExtensionFilter;
import org.xquark.servlet.util.PostDataParser;
import org.xquark.xml.xdbc.*;

public class XQBridgeServlet extends HttpServlet {

    private static final String JNDI_SOURCE = "java:comp/env/jdbc/TestDB";

    private XQBridge bridge;

    private SAXParserFactory factory;

    private HashMap mappings;

    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        factory = SAXParserFactory.newInstance();
        factory.setNamespaceAware(true);
        bridge = new XQBridge("jndi:" + JNDI_SOURCE);
        mappings = loadMappingsFromFiles(config.getServletContext().getRealPath("/WEB-INF/mappings"));
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        PrintWriter out = response.getWriter();
        XMLConnection conn = null;
        try {
            String query = request.getParameter("exec");
            if (query != null) {
                conn = bridge.getXMLConnection();
                execQuery(conn, query, out);
            } else {
                reportError(response, "Unrecognized command: <B>" + request.getContextPath() + request.getServletPath() + request.getQueryString() + "</B>.", "query.jsp");
                return;
            }
        } catch (XMLDBCException e) {
            reportException(response, e, "query.jsp");
        } finally {
            out.close();
            if (conn != null)
                try {
                    conn.close();
                } catch (XMLDBCException e) {
                    // ignore
                }
        }
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        PrintWriter out = response.getWriter();
        String mapping = null;
        String file = null;
        if (request.getContentType().startsWith("multipart/form-data;")) {
            PostDataParser poster = new PostDataParser(request);
            mapping = poster.getParameter("mapping");
            file = poster.getFileContent("file");
        } else if (request.getContentType().equals("application/x-www-form-urlencoded")) {
            mapping = request.getParameter("mapping");
            file = request.getParameter("file");
        } else {
            reportError(response, "Unknown content type: " + request.getContentType(), "insert.jsp");
            return;
        }
        if (mapping == null || file == null) {
            reportError(response, "Missing parameters for file insertion", "insert.jsp");
        } else {
            Mapping map = null;
            Mapper mapper = null;
            try {
                String mapFile = (String) mappings.get(mapping);
                map = bridge.getMapping(mapFile);
                mapper = map.getMapper();
                mapper.setDocumentBase("file:/c:/");
                mapper.insertDocument(file);
                reportMessage(response, "Document inserted successfully.", "insert.jsp");
            } catch (SAXException ex) {
                if (ex.getException() != null)
                    reportException(response, ex.getException(), "insert.jsp");
                else
                    reportException(response, ex, "insert.jsp");
            } catch (Exception ex) {
                reportException(response, ex, "insert.jsp");
            } finally {
                try {
                    if (mapper != null)
                        mapper.close();
                } catch (XMLDBCException e) {
                }
                try {
                    if (map != null)
                        map.close();
                } catch (XMLDBCException e) {
                }
            }
        }
    }

    private HashMap loadMappingsFromFiles(String realPath) {
        HashMap result = new HashMap();
        FileFilter mapFilter = new FileExtensionFilter(".map");
        File mappingsDir = new File(realPath);
        File[] mapDirs = mappingsDir.listFiles();
        for (int i = 0; i < mapDirs.length; i++) {
            File mapDir = mapDirs[i];
            if (mapDir.isDirectory()) {
                File[] mapFiles = mapDir.listFiles(mapFilter);
                if (mapFiles.length > 0) {
                    String mapFile = null;
                    try {
                        mapFile = mapFiles[0].toURL().toString();
                        Mapping mapping = bridge.getMapping(mapFile);
                        result.put(mapDir.getName(), mapFile);
                    } catch (MalformedURLException ex) {
                        log("Error while loading mapping file: " + mapFile, ex);
                    } catch (XMLDBCException ex) {
                        log("Error while loading mapping file: " + mapFile, ex);
                    }
                }
            }
        }
        return result;
    }

    private void execQuery(XMLConnection conn, String query, PrintWriter out) throws ServletException, XMLDBCException {
        XMLStatement stmt = null;
        XMLResultSet xrs = null;
        try {
            stmt = conn.createStatement();
            xrs = stmt.executeQuery(query);
            XMLSerializer serializer = new XMLSerializer(out, "iso-8859-1");
            xrs.setContentHandler(serializer);
            xrs.setLexicalHandler(serializer);
            serializer.startDocument();
            //serializer.processingInstruction("xml-stylesheet", "href=\"xsl/beautify.xsl\" type=\"text/xsl\"");
            serializer.startElement("", "results", "results", new AttributesImpl());
            while (xrs.hasNext())
                xrs.nextAsSAX();
            serializer.endElement("", "results", "results");
            serializer.endDocument();
        } catch (SAXException e) {
            throw new ServletException("Error while producing document result", e);
        } finally {
            if (xrs != null)
                xrs.close();
            if (stmt != null)
                stmt.close();
        }
    }

    private void reportMessage(HttpServletResponse response, String message, String url) throws IOException {
        response.sendRedirect(response.encodeRedirectURL("info.jsp?url=" + url + "&message=" + message));
    }

    private void reportError(HttpServletResponse response, String error, String url) throws IOException {
        response.sendRedirect(response.encodeRedirectURL("error.jsp?url=" + url + "&message=" + error));
    }

    private void reportException(HttpServletResponse response, Exception e, String url) throws IOException {
        /*
         * StringWriter sw = new StringWriter(); e.printStackTrace(new PrintWriter(sw)); String message = " <pre> "+sw.toString()+" </pre> "; response.sendRedirect(response.encodeRedirectURL("exception.jsp?url="+url+"&message="+message));
         */
        PrintWriter out = response.getWriter();
        out.println("<html><body><pre>");
        e.printStackTrace(out);
        out.println("</pre></body></html>");
    }
}